; Hello Sprites 1
; Created by Billy D. Spelchan for Blazing Games Development Blog
;
; A block of sprites saying Hello Sprites!  will bounce around a checkerboard 
; screen to demonstrate using sprites. This version of the program will set the
; sprite data manually.

; iNES header for setting up ROM file
	.inesprg 1   ; 1x 16KB PRG code
	.ineschr 1   ; 1x  8KB CHR data
	.inesmap 0   ; mapper 0 = NROM, no bank swapping
	.inesmir 0   ; horizontal background mirroring

; ***** CONSTANTS

; ***** VARIABLES
	.rsset $0  ; set the initial value of RS counter
SPRITE_X	.rs 1
SPRITE_Y	.rs 1
H_ADJUST	.rs 1
V_ADJUST	.rs 1
TEMP_X		.rs 1
TEMP_Y		.rs 1

; first 8k PRG block 

	.bank 0
	.org $C000 

RESET:
	SEI          ; disable IRQs
	CLD          ; disable decimal mode
	LDX #$40
	STX $4017    ; disable APU frame IRQ
	LDX #$FF
	TXS          ; Set up stack
	INX          ; now X = 0
	STX $2000    ; disable NMI
	STX $2001    ; disable rendering
	STX $4010    ; disable DMC IRQs
	
	jsr WaitForVBlank	; First wait for vblank to make sure PPU ready
	; can do a bit of additional initialization here if necessary
	jsr WaitForVBlank	; Second vblank. PPU now ready
  
LoadPalettes:
	LDA $2002	; read PPU status to reset the high/low latch
	LDA #$3F
	STA $2006	; write the high byte of $3F00 address
	LDA #$00
	STA $2006	; write the low byte of $3F00 address
	LDX #$00	; start out at 0
LoadPalettesLoop:
	LDA palette, x	; load data from address (palette + the value in x)
	STA $2007	; write to PPU
	INX		; X = X + 1
	CPX #$20	; Compare X to hex $10, decimal 16 - copying 16 bytes = 4 sprites
	BNE LoadPalettesLoop  ; Branch to LoadPalettesLoop if compare was Not Equal to zero
			; if compare was equal to 32, keep going down

HelloSprites:
	LDA #$00
	STA $2001    ; disable rendering

ClearScreen:
	LDY #$04
	LDX #$40
	;  LDA #$00
	LDA $2002	; read PPU status to reset the high/low latch
	LDA #$20
	STA $2006	; write the high byte of screen address
	LDA #$00
	STA $2006	; write the low byte of screen address

  	LDA #$15	; checkerboard character
clearScreenLoop:
	STA $2007
	INX
	BNE clearScreenLoop
	DEY
	BNE clearScreenLoop
; fill out attribute table
	LDX #$40
	LDA #$00
clearScreen_attributeFill:
	STA $2007
	DEX
	BNE clearScreen_attributeFill
	
	; initialize the sprites
	LDA #0
	TAY
	STA $2003
	; fill sprite memory with data from ROM
InitSprites_dataLoop:
	LDA hello_sprite_data,Y
	STA $2004
	INY
	CPY #52
	BNE InitSprites_dataLoop
	
	; fill rest of sprite memory with 255 so that it will not be visible
	LDA #255 
InitSprites_fillLoop:
	STA $2004
	INY
	BNE InitSprites_fillLoop
  
	; set sprite variables 
	LDA #0
	STA SPRITE_X
	STA SPRITE_Y
	LDA #1
	STA H_ADJUST
	STA V_ADJUST
	
	; set scrolling position and screen enable screen
	STA $2005
	STA $2005
	LDA #%00011110   ; enable background + sprites
	STA $2001

MainGameLoop:
;	NOP	; Right now we do nothing in the main loop.
	JSR WaitForVBlank
	JSR MoveSprites
	JMP MainGameLoop

; ---------------------------
; Functions  
WaitForVBlank:
	BIT $2002
	BPL WaitForVBlank
	RTS

PoorEmulatorWaitForVBlankEnd:
	BIT $2002
	BMI PoorEmulatorWaitForVBlankEnd
	RTS

; ***** MoveSprites
;
; adjusts sprite positions and bouncing off edges
; Uses X,Y, and A
; Variables used: SPRITE_X, SPRITE_Y, H_ADJUST, V_ADJUST, TEMP_X, TEMP_Y
;
MoveSprites:
	; First adjust top corner of sprite block
	LDA SPRITE_X
	CLC
	ADC H_ADJUST
	STA SPRITE_X
	; Has hit left edge?
	BNE MoveSprite_checkRightEdge
	; if so set horizontal adjust to +1
	LDA #1
	STA H_ADJUST
MoveSprite_checkRightEdge:
	; if has hit right edge?
	CMP #192
	BNE MoveSprite_vertical
	; if hit right edge, set horizontal adjust to -1
	LDA #$FF	;	-1
	STA H_ADJUST
MoveSprite_vertical:
	; Adjust Y coordinate of block
	LDA SPRITE_Y
	CLC
	ADC V_ADJUST
	STA SPRITE_Y
	; has hit top
	CMP #8
	BNE MoveSprite_checkBottomEdge
	; if so set vertical ajdust to +1
	LDA #1
	STA V_ADJUST
MoveSprite_checkBottomEdge:
	; check if hit bottom
	CMP #216
	BNE MoveSprite_adjustSprites
	; if hit bottom edge, set vertical adjust to -1
	LDA #$FF	;	-1
	STA V_ADJUST
	
MoveSprite_adjustSprites:
	; prepare for top row of sprite coordinate adjust
	LDA SPRITE_X
	CLC
	ADC #12
	STA TEMP_X	; adjusted X coordinate
	LDX #5		; X holds count
	LDY #0		; Y holds sprite memory index
MoveSprite_topRowAdjust:
	STY $2003
	LDA SPRITE_Y	; Top row actually at Y coordinate
	STA $2004	; Write Y location
	INY		; Adjust index to point to sprite X data
	INY
	INY
	STY $2003	; tell PPU we want to write sprite X
	LDA TEMP_X
	STA $2004	; Write sprite X coordinate
	CLC
	ADC #8		; add 8 to this coordinate for next sprite
	STA TEMP_X
	INY		; set up index for next sprite
	DEX		; and adjust countdown
	BNE MoveSprite_topRowAdjust
	; we reach here when top row done, so prepare bottom row
	LDA SPRITE_Y
	CLC
	ADC #8
	STA TEMP_Y	; Y coordinate to use for bottom row of sprites
	LDA SPRITE_X
	STA TEMP_X	; X coordinate starts at block x coordinate
		; Y register already correct so no need to set
	LDX #8		; 8 sprites in bottom row need to be counted
MoveSprite_bottomRowAdjust:
	STY $2003
	LDA TEMP_Y
	STA $2004	; write adjusted sprite Y coordinate
	INY		; Adjust index to point to sprite X data
	INY
	INY
	STY $2003	; tell PPU we want to write sprite X
	LDA TEMP_X
	STA $2004	; Write sprite X coordinate
	CLC
	ADC #8		; add 8 to this coordinate for next sprite
	STA TEMP_X
	INY		; set up index for next sprite
	DEX		; and adjust countdown
	BNE MoveSprite_bottomRowAdjust
	
	RTS	
; ---------------------------
; Game Data

	.bank 1
	.org $E000
palette:
	.db $0F,$19,$2B,$39, $0F,$17,$16,$06, $0F,$39,$3A,$3B, $0F,$3D,$3E,$0F
	.db $0F,$14,$27,$39, $0F,$27,$06,$2D, $0F,$0A,$29,$25, $0F,$02,$38,$3C

hello_sprite_data:
	;     Y, char, flags, X 
	.db 0, 'H', 0, $0C
	.db 0, 'e', 0, $14
	.db 0, 'l', 0, $1C
	.db 0, 'l', 0, $24
	.db 0, 'o', 0, $2C
	
	.db 8, 'S', 1, $00
	.db 8, 'p', 1, $08
	.db 8, 'r', 1, $10
	.db 8, 'i', 1, $18
	.db 8, 't', 1, $20
	.db 8, 'e', 1, $28
	.db 8, 's', 1, $30
	.db 8, '!', 1, $38
  
; jump Vectors
	.org $FFFA     ;first of the three jump vectors starts here
	.dw 0          ; Jump for NMIs if enabled;
	.dw RESET      ; jump for when RESET or first turned on
	.dw 0          ;external interrupt IRQ is not used
  
; ---------------------------
; Character Tables (tilesets)
	.bank 2
	.org $0000
	.incbin "tileset.chr"   ;includes 8KB graphics file from SMB1
  
